// Melodieklingel   -   Guido Scheidat
// Nachbau der Melodieklingel aus FA 4/86 mit modernen Bauteilen
// bernahme aller Melodien aus der "Funkamateur"

// ATmega8 
// intern 8 MHz
// Pin 4 (PD2 / INT0) - Klingeltaster
// Pin 15 (PB1 / OC1A) - Ausgang zum Lautsprecher

// avr-gcc (GCC) 4.2.2 (WinAVR 20071221)


#include <inttypes.h>		// uint8_t usw.
#include <avr/interrupt.h>	// Interrupts
#include <avr/pgmspace.h>
#include <util/delay.h>		// definiert _delay_ms und _delay_loop_2 

#include "main.h"
#include "ton.h"			//Melodien, Offsets und Tonhhen
#include "lied.h"           //Melodien, Offsets und Tonhhen


uint8_t  viertel;
uint16_t liedcount;
uint8_t  aktton;
uint8_t  lang;
uint16_t note;
uint8_t  status;  
uint8_t  i;
uint8_t  speed;


ISR(INT0_vect)
{
    GICR &= ~(1 << INT0); // INT0: External Interrupt Request 0 Disable

	// Taste muss mindestens 30 ms gedrckt sein
    i = 0;
    while (i < 30)
    {
        if (!(PIND & TASTE))
		{
			_delay_ms(1); // Pause
			i++;
		}
        else i = 0;
    }

	// Taste muss mindestens 30 ms losgelassen sein
    i = 0;
    while (i < 30)
    {
        if (PIND & TASTE)
		{
            _delay_ms(1); // Pause
			i++;
		}
        else i = 0;
    }

	TIMSK |= (1 << TOIE0); // Timer/Counter0 Overflow Interrupt Enable
    viertel = 1;
    lang = 1;
    status &= ~(1 << enderr); // = 0;

	SOUNDON;
}


ISR(TIMER0_OVF_vect)     //Interrupt Zeitsteuerung
{
	if (--viertel == 0)
	{
		viertel = speed;
		
		if (--lang == 0)
		{
			if (status & (1 << enderr))
			{
				// Timer/Counter0 Overflow Interrupt disable
				TIMSK &= ~(1 << TOIE0);
				SOUNDOFF;
                GIFR |= (1 << INTF0); // External Interrupt Flag 0 set
                GICR |= (1 << INT0);
			}
            else
            {
                aktton = pgm_read_byte(&lied[liedcount]);
				
                if ((aktton & 0x80) == 0x80) status |= (1 << enderr);
                lang = (aktton >> 4) & 0x07;
                lang++;
				aktton = aktton & 0x0F;
                if (aktton == 0)
                {
                    SOUNDOFF;
                }
                else
                {
                    aktton--;
                    TCCR1B &= ~(1 << CS11) & ~(1 << CS12) &  ~(1 << CS10);
                    
                    note = pgm_read_word(&tone[aktton]);
                    
                    OCR1AH = (unsigned char)(note >> 8);
                    OCR1AL = (unsigned char)note;
                    
                    TCNT1H = 0;
                    TCNT1L = 0;
                    
                    TCCR1B |=  (1 << CS10); 
                    
                    SOUNDON;
					
					status &= ~(1 << pause);
                }
                liedcount++;
                if (pgm_read_byte(&lied[liedcount]) == 0xFF) liedcount = 0;
            }
		}
	}
}


int main(void)
{
	liedcount = 0;
	lang = 1;
	status &= ~(1 << enderr);
	viertel = 1;
	speed = 4;

	PORTD |= TASTE; //Pull up an

	SOUNDOFF;

// -----------------------------------------------------------------------------------------------------
	// Timer 1 initialisieren

	// Vorteiler auf 1
	TCCR1B |=  (1 << CS10); 
	TCCR1B &= ~(1 << CS11) & ~(1 << CS12);

	// Toggle OC1A on Compare Match
	TCCR1A &= ~(1 << COM1A1);
	TCCR1A |= (1 << COM1A0);

	// Waveform Generation Mode Bit Description --> Mode 4; CTC
	// WGM13 = 0
	// WGM12 = 1
	// WGM11 = 0
	// WGM10 = 0
	TCCR1B &= ~(1 << WGM13);
	TCCR1B |= (1 << WGM12);
	TCCR1A &= ~(1 << WGM10) & ~(1 << WGM11);


// -----------------------------------------------------------------------------------------------------
	// Timer 0 initialisieren

	// Vorteiler einstellen auf 1024
	// 	CS02	CS01	CS00
	//	  1		  0		  1
	TCCR0 |= (1 << CS02) | (1 << CS00);
	TCCR0 &= ~(1 << CS01);
	// Timer/Counter
	TCNT0 = 256-243; // entspricht 1/8 / 4
	// Timer/Counter0 Overflow Interrupt Enable
	TIMSK |= (1 << TOIE0);
	// Timer/Counter0 Overflow Flag
	TIFR |= (1 << TOV0);

// -----------------------------------------------------------------------------------------------------
    // INT 0 initialisieren

    // The falling edge of INT0 generates an interrupt request.
    MCUCR |= (1 << ISC01);
    MCUCR &= ~(1 << ISC00);
    
    GICR |= (1 << INT0); // INT0: External Interrupt Request 0 Enable

// -----------------------------------------------------------------------------------------------------

	sei(); //Interrupts ein

	while(1)
	{
	}
} //main
